;Routinen zur Ausgabe von Binrzahlen
;von 8, 16, 24 und 32Bit auf einem LCD.
;Input:
;Z mit der Adresse der Binrzahl im SRAM.
;x(xh:xl)=X-Position (Spalte) der Zahl auf dem LCD.
;r17=Y-Position (Zeile) der Zahl auf dem LCD.
;BCDFlag (r19)=Anzahl der Kommastellen im unteren Nibble und an Bit 4 = 1/0
;fr Print Using aus/an.

.def BCDVal0=r12		;Wert fr 8Bit und low-Byte fr 16, 24 und 32Bit.
.def BCDVal1=r13		;High-Byte fr 16Bit und 2. Byte fr 24 und 32Bit.
.def BCDVal2=r14		;High-Byte fr 24Bit und 3. Byte fr 32Bit.
.def BCDVal3=r15		;High-Byte fr 32Bit.
.def BCDChar=r16		;Auszugebendes Zeichen.
.def BCDFlag=r19		;Flag fr Anzahl der Kommastellen im unteren Nibble
							;und Bit 4 = 0 fr Using (rechts bndig).
.def BCDDiv0=r20		;Divisor unteres Byte.
.def BCDDiv1=r21		;Divisor 2. Byte.
.def BCDDiv2=r22		;Divisor 3. Byte.
.def BCDDiv3=r23		;Divisor 4. Byte.
.def BCD0Char=r24		;Ersatzzeichen fr die fhrenden Nullen (Leerzeichen).
.def BCDPCount=r25	;Counter fr Kommastellen.

;Ausgabe von 8Bit-Zahlen mit max. 2 Kommastellen.
PrintBCD8:
	ld		BCDVal0,z		;8Bit-Wert aus Z in Value 0 laden.
	ldi	BCD0Char,' '	;Ersatzzeichen ' ' (Leerzeichen) setzen.
	ldi	BCDPCount,3		;Max. Anzahl Kommastellen +1.
	ldi	BCDDiv0,100		;Divior 0 laden.
	rcall	BCD8Sub			;Value 0 durch Divisor 0 teilen und 1. Zeichen/Ziffer ausgeben.
	rjmp	BCD8J				;In der BCD32-Routine geht es weiter.

;Ausgabe von 16Bit-Zahlen mit max. 4 Kommastellen.
PrintBCD16:
	ld		BCDVal0,z					;Low-Byte fr 16Bit aus Z in Value 0 laden.
	ldd	BCDVal1,z+1					;High-Byte fr 16Bit aus Z in Value 1 laden.
	ldi	BCD0Char,' '				;Ersatzzeichen ' ' (Leerzeichen) setzen.
	ldi	BCDPCount,5					;Max. Anzahl Kommastellen +1.
	ldiw	BCDDiv0,BCDDiv1,10000	;Divior 0 und 1 laden.
	rcall	BCD16Sub						;Value 0 und 1 durch Divisor 0 und 1 teilen und 1. Zeichen/Ziffer ausgeben.
	rjmp	BCD16J						;In der BCD32-Routine geht es weiter.

;Ausgabe von 24Bit-Zahlen mit max. 7 Kommastellen.
PrintBCD24:
	ld		BCDVal0,z
	ldd	BCDVal1,z+1
	ldd	BCDVal2,z+2
	ldi	BCD0Char,' '
	ldi	BCDPCount,8
	ldi24	BCDDiv0,BCDDiv1,BCDDiv2,10000000
	rcall	BCD24Sub
	rjmp	BCD24J

;Ausgabe von 32Bit-Zahlen mit max. 9 Kommastellen.
;Hier sind fr PrintBCD8, 16 und 24 verschiedene Einsprungpunkte.
PrintBCD32:
	ld		BCDVal0,z
	ldd	BCDVal1,z+1
	ldd	BCDVal2,z+2
	ldd	BCDVal3,z+3
	ldi	BCD0Char,' '
	ldi	BCDPCount,10
	ldi32	BCDDiv0,BCDDiv1,BCDDiv2,BCDDiv3,1000000000
	rcall	BCD32Sub
	ldi32	BCDDiv0,BCDDiv1,BCDDiv2,BCDDiv3,100000000
	rcall	BCD32Sub
	ldi32	BCDDiv0,BCDDiv1,BCDDiv2,BCDDiv3,10000000
	rcall	BCD32Sub
BCD24J:
	ldi24	BCDDiv0,BCDDiv1,BCDDiv2,1000000
	rcall	BCD24Sub
	ldi24	BCDDiv0,BCDDiv1,BCDDiv2,100000
	rcall	BCD24Sub
	ldi24	BCDDiv0,BCDDiv1,BCDDiv2,10000
	rcall	BCD24Sub
BCD16J:
	ldiw	BCDDiv0,BCDDiv1,1000
	rcall	BCD16Sub
	ldiw	BCDDiv0,BCDDiv1,100
	rcall	BCD16Sub
BCD8J:
	ldi	BCDDiv0,10
	rcall	BCD8Sub
	mov	BCDChar,BCDVal0
	subi	BCDChar,-'0'
	rjmp	PrintCode

;Teilt einen 8Bit-Wert durch einen 8Bit Divisor
;und springt zur Zeichenausgabe.
BCD8Sub:
	ldi	BCDChar,'0'-1
BCD8Sub1:
	inc	BCDChar
	sub	BCDVal0,BCDDiv0
	brcc	BCD8Sub1
	add	BCDVal0,BCDDiv0
	rjmp	BCD32Sub2

;Teilt einen 16Bit-Wert durch einen 16Bit Divisor
;und springt zur Zeichenausgabe.
BCD16Sub:
	ldi	BCDChar,'0'-1
BCD16Sub1:
	inc	BCDChar
	subw	BCDVal0,BCDVal1,BCDDiv0,BCDDiv1
	brcc	BCD16Sub1
	addw	BCDVal0,BCDVal1,BCDDiv0,BCDDiv1
	rjmp	BCD32Sub2

;Teilt einen 24Bit-Wert durch einen 24Bit Divisor
;und springt zur Zeichenausgabe.
BCD24Sub:
	ldi	BCDChar,'0'-1
BCD24Sub1:
	inc	BCDChar
	sub24	BCDVal0,BCDVal1,BCDVal2,BCDDiv0,BCDDiv1,BCDDiv2
	brcc	BCD24Sub1
	add24	BCDVal0,BCDVal1,BCDVal2,BCDDiv0,BCDDiv1,BCDDiv2
	rjmp	BCD32Sub2

;Teilt einen 32Bit-Wert durch einen 32Bit Divisor.
;Anschlieend erfolgt die Zeichenausgabe.
BCD32Sub:
	ldi	BCDChar,'0'-1
;Hier solange mit Divisor 0 - 3 subtrahieren bis berlauf.
BCD32Sub1:
	inc	BCDChar
	sub32	BCDVal0,BCDVal1,BCDVal2,BCDVal3,BCDDiv0,BCDDiv1,BCDDiv2,BCDDiv3
	brcc	BCD32Sub1
;Zur korrektur den 32Bit-Wert mit den Divisor addieren.
	add32	BCDVal0,BCDVal1,BCDVal2,BCDVal3,BCDDiv0,BCDDiv1,BCDDiv2,BCDDiv3
;Zeichenausgabe.
;Feststellen, ob das Komma ausgegeben werden mu.
BCD32Sub2:
	dec	BCDPCount
	mov	r18,BCDFlag
	andi	r18,0x0f
	cp		r18,BCDPCount
	brne	BCD32Sub3
	rcall	PrintCode
	ldi	BCD0Char,'0'
	ldi	BCDChar,','
	rjmp	PrintCode
;Prfen, ob eine 0 als 0 oder durch ein Ersatzzeichen ' ' ausgegeben werden mu.
BCD32Sub3:
	cpi	BCDChar,'0'
	breq	BCD32Sub4
	ldi	BCD0Char,'0'
	rjmp	PrintCode
BCD32Sub4:
	mov	BCDChar,BCD0Char
	cpi	BCDChar,' '
	brne	BCD32Sub5
	sbrc	BCDFlag,4
	ret
BCD32Sub5:
	rjmp	PrintCode
